package com.ebei.screen.common.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

import java.text.NumberFormat;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * EBA大屏工具类
 *
 * @author Levi.u
 * @date 2021/1/13 21:03
 */
@Slf4j
@Component
public class EbaUtils {

    private static Integer SUCCESS_CODE = 200;
    private static MongoTemplate mongoTemplate;
    private static List<Map> initEmptyList;
    public static String token;
    public static JSONObject project;
    public static JSONObject deviceSystem;
    public static JSONObject deviceType;
    public static JSONObject deviceParam;
    public static JSONObject deviceAlarmType;
    public static JSONObject fireDeviceType;
    public static JSONObject realDataDevice;

    static {
        // 初始化模板
        List<Map> monthList = new ArrayList<>();
        Integer monthDays = LeviUtils.getMonthDays(LocalDateTime.now().getMonthValue());
        for (int b = 1; b <= monthDays; b++) {
            List<Map> dayList = new ArrayList<>();
            for (int c = 0; c <= 23; c++) {
                dayList.add(Levi.by("hour", c).set("data", new ArrayList<>()));
            }
            monthList.add(Levi.by("day", b).set("data", dayList));
        }
        initEmptyList = monthList;
    }

    @Autowired
    public void setMongoTemplate(MongoTemplate mongoTemplate) {
        EbaUtils.mongoTemplate = mongoTemplate;
    }

    /**
     * 判断执行结果是否成功
     *
     * @param result true成功 false失败
     * @return
     */
    public static boolean isSuccess(String result) {
        return SUCCESS_CODE.equals(JSON.parseObject(result).getInteger("code"));
    }

    /**
     * 用于存储数据(不带统计数字)
     *
     * @param obj            插入的对象
     * @param pkId           区分键
     * @param collectionName 集合名字
     * @param flag           true 覆盖原有数据只保留一条最新的 false追加原有数据
     * @return
     */
    public static List<Map> setDataField(Object obj, String pkId, String collectionName, boolean flag) {
        LocalDateTime now = LocalDateTime.now();
        int year = now.getYear();
        int month = now.getMonthValue();
        int day = now.getDayOfMonth();
        int hour = now.getHour();
        Map monthMap = mongoTemplate.findById(pkId + "_" + year + "_" + month, Map.class, collectionName);
        if (MapUtil.isEmpty(monthMap)) {
            monthMap = Levi.by("_id", pkId + "_" + year + "_" + month).set("data", LeviUtils.deepCopy(initEmptyList));
        }
        List<Map> dayList = (List<Map>) monthMap.get("data");
        Map dayMap = dayList.get(day - 1);
        List<Map> hourList = (List<Map>) dayMap.get("data");
        Map hourMap = hourList.get(hour);
        List<Map> dataList = (List<Map>) hourMap.get("data");
        if (CollUtil.isNotEmpty(dataList) && !flag) {
            dataList.addAll((List<Map>) obj);
            hourMap.put("data", dataList);
        } else {
            hourMap.put("data", obj);
        }
        return dayList;
    }

    /**
     * 用于存储数据(带统计数字)
     *
     * @param key            count字段key名称
     * @param obj            插入的对象
     * @param pkId           区分键
     * @param collectionName 集合名字
     * @param flag           true 覆盖原有数据只保留一条最新的 false追加原有数据
     * @return
     */
    public static Map setDataFieldCount(String key, Object obj, String pkId, String collectionName, boolean flag) {
        LocalDateTime now = LocalDateTime.now();
        int year = now.getYear();
        int month = now.getMonthValue();
        int day = now.getDayOfMonth();
        int hour = now.getHour();
        List<Map> objList = (List<Map>) obj;
        Map monthMap = mongoTemplate.findById(pkId + "_" + year + "_" + month, Map.class, collectionName);
        if (MapUtil.isEmpty(monthMap)) {
            monthMap = Levi.by("_id", pkId + "_" + year + "_" + month).set("data", LeviUtils.deepCopy(initEmptyList));
        }
        List<Map> dayList = (List<Map>) monthMap.get("data");
        Map dayMap = dayList.get(day - 1);
        List<Map> hourList = (List<Map>) dayMap.get("data");
        Map hourMap = hourList.get(hour);
        List<Map> dataList = (List<Map>) hourMap.get("data");
        if (CollUtil.isNotEmpty(dataList) && !flag) {
            dataList.addAll(objList);
            hourMap.put("data", dataList);
            int size = objList.size();
            hourMap.put(key, addCount(hourMap, size, key));
            dayMap.put(key, addCount(dayMap, size, key));
            monthMap.put(key, addCount(monthMap, size, key));
        } else {
            hourMap.put("data", obj);
            int size = objList.size();
            hourMap.put(key, addCount(hourMap, size, key) - dataList.size());
            dayMap.put(key, addCount(dayMap, size, key) - dataList.size());
            monthMap.put(key, addCount(monthMap, size, key) - dataList.size());
        }
        return monthMap;
    }

    private static Integer addCount(Map a, Integer b, String key) {
        Integer count = (Integer) a.get(key);
        return count == null ? b : (count + b);
    }

    /**
     * 获取同比数据
     *
     * @param old 去年数
     * @param now 今年数
     * @return
     */
    public static String getTongBi(Integer old, Integer now) {
        float oldF = (float) old;
        float nowF = (float) now;
        if (old == null || old == 0) {
            return "100%";
        }
        NumberFormat format = NumberFormat.getPercentInstance();
        float result = (nowF - oldF) / oldF;
        return format.format(result);
    }
}
