package com.ebei.screen.service;

import com.ebei.screen.common.response.ResponseBean;

import java.util.Map;

/**
 * EBA系统大屏
 *
 * @author Levi.u
 * @date 2021/1/13 14:26
 */
public interface EbaSystemService {

    /**
     * 初始化数据
     */
    void initData();

    /**
     * 初始化当前用户名token到mongo
     */
    void initToken();

    /**
     * 初始化当前用户名下项目列表到mongo
     */
    void initProjectList();

    /**
     * 初始化设备信息(deviceSys)到mongo
     */
    void initDeviceSystemList();

    /**
     * 初始化设备类型信息(deviceType)到mongo
     */
    void initDeviceTypeList();

    /**
     * 初始化设备参数信息(deviceParam)到mongo
     */
    void initDeviceParamList();

    /**
     * 初始化设备告警类型信息(alarmType)到mongo
     */
    void initDeviceAlarmTypeList();

    /**
     * 初始化消防设备类型信息(fireDeviceType)到mongo
     */
    void initFireDeviceTypeList();

    /**
     * 初始化所有项目实时device信息到mongo
     */
    void initRealDataDeviceList();

    /**
     * 初始化所有项目device信息到mongo
     */
    void initMobjectList();

    /**
     * 初始化所有项目iot信息到mongo
     */
    void initIotList();

    /**
     * 实时数据回调接口对查
     *
     * @param params
     * @return
     */
    Integer realTimeCallback(Map<String, String> params);

    /**
     * 获取项目列表
     *
     * @return
     */
    ResponseBean getProjectList();

    /**
     * 获取实时设备列表
     *
     * @param projectId 项目id
     * @return
     */
    ResponseBean getDeviceList(String projectId);

    /**
     * 获取实时火警和预警
     *
     * @param projectId 项目id
     * @return
     */
    ResponseBean getRealTimeList(String projectId);
}
