package com.ebei.screen.service.impl;

import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.common.util.CommonUtil;
import com.ebei.screen.modules.po.MachineRoom;
import com.ebei.screen.service.MachineRoomService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.regex.Pattern;

/**
 * @version v1.0
 * @ProjectName: screen
 * @ClassName: MachineRoomServiceImpl
 * @Description:
 * @Author: ch
 * @Date: 2021-01-20 14:07
 */
@Slf4j
@Service
@AllArgsConstructor
public class MachineRoomServiceImpl implements MachineRoomService {

    @Autowired
    private MongoTemplate mongoTemplate;

    @Override
    public void saveMachineRoom(MachineRoom machineRoom) {
        LocalDateTime now = LocalDateTime.now();
        machineRoom.setId(CommonUtil.getSnowId().toString());
        machineRoom.setCreateTime(now);
        machineRoom.setType(machineRoom.getType());
        machineRoom.setIsDelete(0);
        mongoTemplate.insert(machineRoom);
    }

    @Override
    public void updateMachineRoom(MachineRoom machineRoom) {
        LocalDateTime now = LocalDateTime.now();
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is(machineRoom.getId()));
        Update update = new Update();
        update.set("name", machineRoom.getName());
        update.set("projectId", machineRoom.getProjectId());
        update.set("type", machineRoom.getType());
        update.set("cameraDeviceList", machineRoom.getCameraDeviceList());
        update.set("updateTime", now);
        update.set("isDelete", machineRoom.getIsDelete());
        mongoTemplate.updateFirst(query, update, MachineRoom.class);
    }

    @Override
    public ResponseBean queryMachineRoom(int page, Integer pageSize, String name, String projectId) {
        Query query = new Query();
        if (CommonUtil.isNotEmpty(name)) {
            Criteria criteria = new Criteria();
            Pattern pattern = Pattern.compile("^.*" + name + ".*$", Pattern.CASE_INSENSITIVE);
            criteria.where("name").regex(pattern);
            query.addCriteria(criteria);
        }
        if (CommonUtil.isNotEmpty(projectId)) {
            query.addCriteria(Criteria.where("projectId").is(projectId));
        }
        query.addCriteria(Criteria.where("isDelete").is(0));
        //计算总数
        long total = mongoTemplate.count(query, MachineRoom.class, "MachineRoom");
        //分页查询
        Pageable pageable = PageRequest.of(page, pageSize);
        query.with(pageable);
        query.with(Sort.by(Sort.Direction.DESC, "createTime"));
        List<MachineRoom> result = mongoTemplate.find(query, MachineRoom.class);
        Page<MachineRoom> pageResult = new PageImpl<>(result, pageable, total);
        return ResponseBean.success(pageResult);
    }
}
